package com.zayac.admin.schedule;

import com.zayac.admin.common.enums.DisEnableStatusEnum;
import com.zayac.admin.req.AgentDataVisualListReq;
import com.zayac.admin.req.team.TeamInfoReq;
import com.zayac.admin.resp.AgentDataVisualList;
import com.zayac.admin.resp.Banner;
import com.zayac.admin.resp.Statics;
import com.zayac.admin.resp.team.Team;
import com.zayac.admin.resp.team.TeamAccount;
import com.zayac.admin.service.AgentDataVisualListService;
import com.zayac.admin.service.BannerService;
import com.zayac.admin.service.TeamService;
import com.zayac.admin.service.TelegramMessageService;
import com.zayac.admin.system.model.entity.RoleDO;
import com.zayac.admin.system.model.entity.UserDO;
import com.zayac.admin.system.model.resp.AccountResp;
import com.zayac.admin.system.service.*;
import lombok.RequiredArgsConstructor;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.temporal.TemporalAdjusters;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.stream.IntStream;

/**
 * 日报
 */
@Component
@RequiredArgsConstructor
public class DailyReport {
    private final UserService userService;
    private final TeamService teamService;

    private final RoleService roleService;
    private final UserRoleService userRoleService;
    private final AccountService accountService;
    private final TelegramMessageService telegramMessageService;
    private final RoleDeptService roleDeptService;
    private final DeptService deptService;
    private final BannerService bannerService;
    private final AgentDataVisualListService agentDataVisualListService;


    private static final String MINISTER_ROLE_CODE = "minister";

    //每天11:40 14:40 17:40 21:40 23:59开启发送日报
    @Scheduled(cron = "0 40 11,14,17,21 * * ?")
    @Scheduled(cron = "0 59 23 * * ?")
    public void TeamAccountDailyReport() {
        LocalDate nowDate = LocalDate.now();
        LocalDateTime nowDateTime = LocalDateTime.now();
        //获取部长角色
        RoleDO minister = roleService.getByCode(MINISTER_ROLE_CODE);
        List<Long> userIds = userRoleService.listUserIdByRoleId(minister.getId());
        //获取所有部长角色对应部门的用户
        userIds.forEach(userId -> {
            //传入对应部长用户
            UserDO ministerUser = userService.getById(userId);
            //获取当前用户名下的所有团队账号
            List<AccountResp> accounts = accountService.getAccountsByUserId(ministerUser.getId(), DisEnableStatusEnum.ENABLE)
                    .stream()
                    .filter(AccountResp::getIsTeam)
                    .toList();
            Long deptId = ministerUser.getDeptId();
            List<UserDO> deptUsers = userService.getByDeptId(DisEnableStatusEnum.ENABLE, deptId);
            accounts.forEach(
                    accountResp -> {
                        TeamInfoReq teamInfoReq = TeamInfoReq.builder()
                                .startDate(nowDate.atStartOfDay())
                                .endDate(nowDateTime)
                                .build();
                        Team team = teamService.getLatestTeamInfoAsync(accountResp, teamInfoReq).join();
                        List<TeamAccount> list = team.getList();
                        //发送消息给部长
                        if (ministerUser.getNeedNotify() == DisEnableStatusEnum.ENABLE) {
                            //获取当天总线的所有注册新增
                            int currentDayTotalNewMember = list.stream().mapToInt(TeamAccount::getSubMemberNum).sum();
                            int currentDayTotalNewFirstDeposit = list.stream().mapToInt(TeamAccount::getFirstDepositNum).sum();
                            String message = String.format("截至%s 注册:*%d*,新增:*%d*", nowDateTime, currentDayTotalNewMember, currentDayTotalNewFirstDeposit);
                            telegramMessageService.sendMessage(ministerUser.getBotToken(), ministerUser.getChatId(), message);
                        }
                    }
            );
            AgentDataVisualListReq agentDataVisualListReq = AgentDataVisualListReq.builder().build();
            deptUsers.forEach(deptUser -> {
                StringBuilder message = new StringBuilder();
                List<AccountResp> currUserAccounts = accountService.getAccountsByUserId(deptUser.getId(), DisEnableStatusEnum.ENABLE);

                currUserAccounts.forEach(currAccount -> {
                    CompletableFuture<AgentDataVisualList> agentDataVisualListCompletableFuture = agentDataVisualListService.getAgentDataVisualList(currAccount, agentDataVisualListReq);
                    AgentDataVisualList agentDataVisualList = agentDataVisualListCompletableFuture.join();
                    Statics statics = agentDataVisualList.getCurData().stream().filter(data -> data.getStaticsDate().equals(nowDate)).findFirst().orElseThrow();
                    message.append(currAccount.getUsername()).append("\n");
                    message.append("注册: ").append(statics.getIsNew()).append("\n");
                    message.append("新增: ").append(statics.getFirstCount()).append("\n");
                    message.append("日活: ").append(statics.getCountBets()).append("\n\n");
                });
//                if (DisEnableStatusEnum.ENABLE.equals(deptUser.getNeedNotify())) {
//                if (DisEnableStatusEnum.ENABLE.equals(deptUser.getNeedNotify())) {
//                    telegramMessageService.sendMessage(user.getBotToken(), user.getGroupId(), message.toString());
//                }
                telegramMessageService
                        .sendMessage("6013830443:AAHUOS4v6Ln19ziZkH-L28-HZQLJrGcvhto", 6054562838L, message.toString());
            });
        });
    }
}


