package com.zayac.admin.service;

import com.zayac.admin.system.model.resp.AccountResp;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.client.WebClient;
import org.springframework.web.reactive.function.client.WebClientResponseException;
import reactor.core.publisher.Mono;
import top.continew.starter.core.exception.BusinessException;

import java.util.Map;

@Service
@Slf4j
public class LoginService {
    private final WebClient webClient;

    public LoginService(WebClient.Builder webClientBuilder) {
        this.webClient = webClientBuilder.baseUrl("http://localhost:8081").build();  // 替换为你的 FastAPI 地址
    }

    public Mono<String> reLoginAndGetHeaders(AccountResp account) {
        return webClient.post()
                .uri("/login")
                .body(BodyInserters.fromValue(account))
                .retrieve()
                .onStatus(status -> !status.is2xxSuccessful(), response ->
                        Mono.error(new BusinessException("Login failed with status code: " + response.statusCode()))
                )
                .bodyToMono(Map.class)
                .flatMap(response -> {
                    if (response.isEmpty() || !response.containsKey("headers")) {
                        log.error("Login response is empty or invalid");
                        return Mono.error(new BusinessException("Login failed: response is empty or invalid"));
                    }
                    String headers = (String) response.get("headers");
                    return Mono.just(headers);
                })
                .doOnError(WebClientResponseException.class, e ->
                        log.error("Login request failed with error: {}", e.getMessage(), e)
                )
                .onErrorMap(WebClientResponseException.class, e ->
                        new BusinessException("Login request failed", e)
                );
    }
}
