/*
 * Copyright (c) 2022-present Charles7c Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.zayac.admin.service;

import lombok.RequiredArgsConstructor;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;

@Service
@RequiredArgsConstructor
public class TelegramMessageService {
    private final RabbitTemplate rabbitTemplate;

    public void sendMessage(String botToken, Long targetId, String message) {
        String fullMessage = String.format("%s|%s|%s", botToken, targetId, escapeMarkdown(message));
        this.rabbitTemplate.convertAndSend("message_queue", fullMessage);
    }

    private static String escapeMarkdown(String text) {

        List<Character> escapeChars = Arrays
            .asList('_', '[', ']', '(', ')', '~', '>', '#', '+', '-', '=', '|', '{', '}', '.', '!');
        for (Character charItem : escapeChars) {
            text = text.replace(charItem.toString(), "\\" + charItem);
        }
        return text;
    }
}
